#ifndef CAPTURE_IMU_H
#define CAPTURE_IMU_H

//Wolf includes
#include "imu/math/imu_tools.h"
#include "core/capture/capture_motion.h"

namespace wolf {
    
WOLF_PTR_TYPEDEFS(CaptureImu);

class CaptureImu : public CaptureMotion
{
    public:

        CaptureImu(const TimeStamp& _init_ts,
                   SensorBasePtr _sensor_ptr,
                   const Eigen::Vector6d& _data,
                   const Eigen::MatrixXd& _data_cov,
                   CaptureBasePtr _capture_origin_ptr = nullptr);

        CaptureImu(const TimeStamp& _init_ts,
                   SensorBasePtr _sensor_ptr,
                   const Eigen::Vector6d& _data,
                   const Eigen::MatrixXd& _data_cov,
                   const Vector6d& _bias,
                   CaptureBasePtr _capture_origin_ptr = nullptr);

        virtual ~CaptureImu();

        virtual VectorXd correctDelta(const VectorXd& _delta, const VectorXd& _delta_error) const override;

};

inline Eigen::VectorXd CaptureImu::correctDelta(const VectorXd& _delta, const VectorXd& _delta_error) const
{
    return imu::plus(_delta, _delta_error);
}

} // namespace wolf

#endif // CAPTURE_Imu_H
