#ifndef PROCESSOR_IMU_H
#define PROCESSOR_IMU_H

// Wolf
#include "imu/capture/capture_imu.h"
#include "imu/feature/feature_imu.h"
#include "core/processor/processor_motion.h"

namespace wolf {
WOLF_STRUCT_PTR_TYPEDEFS(ProcessorParamsImu);

struct ProcessorParamsImu : public ProcessorParamsMotion
{
    ProcessorParamsImu() = default;
    ProcessorParamsImu(std::string _unique_name, const ParamsServer& _server):
        ProcessorParamsMotion(_unique_name, _server)
    {
        //
    }
    std::string print() const
    {
        return "\n" + ProcessorParamsMotion::print();
    }
};

WOLF_PTR_TYPEDEFS(ProcessorImu);
    
//class
class ProcessorImu : public ProcessorMotion{
    public:
        ProcessorImu(ProcessorParamsImuPtr _params_motion_Imu);
        virtual ~ProcessorImu();
        // virtual void configure(SensorBasePtr _sensor) override { };
        virtual void configure(SensorBasePtr _sensor) override;

        WOLF_PROCESSOR_CREATE(ProcessorImu, ProcessorParamsImu);

    protected:
        virtual void computeCurrentDelta(const Eigen::VectorXd& _data,
                                         const Eigen::MatrixXd& _data_cov,
                                         const Eigen::VectorXd& _calib,
                                         const double _dt,
                                         Eigen::VectorXd& _delta,
                                         Eigen::MatrixXd& _delta_cov,
                                         Eigen::MatrixXd& _jacobian_calib) const override;
        virtual void deltaPlusDelta(const Eigen::VectorXd& _delta_preint,
                                    const Eigen::VectorXd& _delta,
                                    const double _dt,
                                    Eigen::VectorXd& _delta_preint_plus_delta) const override;
        virtual void deltaPlusDelta(const Eigen::VectorXd& _delta_preint,
                                    const Eigen::VectorXd& _delta,
                                    const double _dt,
                                    Eigen::VectorXd& _delta_preint_plus_delta,
                                    Eigen::MatrixXd& _jacobian_delta_preint,
                                    Eigen::MatrixXd& _jacobian_delta) const override;
        virtual void statePlusDelta(const Eigen::VectorXd& _x,
                                const Eigen::VectorXd& _delta,
                                const double _dt,
                                Eigen::VectorXd& _x_plus_delta ) const override;
        virtual Eigen::VectorXd deltaZero() const override;
        virtual bool voteForKeyFrame() const override;
        virtual CaptureMotionPtr emplaceCapture(const FrameBasePtr& _frame_own,
                                                const SensorBasePtr& _sensor,
                                                const TimeStamp& _ts,
                                                const VectorXd& _data,
                                                const MatrixXd& _data_cov,
                                                const VectorXd& _calib,
                                                const VectorXd& _calib_preint,
                                                const CaptureBasePtr& _capture_origin) override;
        virtual FeatureBasePtr emplaceFeature(CaptureMotionPtr _capture_motion) override;
        virtual FactorBasePtr emplaceFactor(FeatureBasePtr _feature_motion,
                                            CaptureBasePtr _capture_origin) override;

    protected:
        ProcessorParamsImuPtr params_motion_Imu_;
        Eigen::Matrix<double, 9, 9> unmeasured_perturbation_cov_;

};

}

/////////////////////////////////////////////////////////
// IMPLEMENTATION. Put your implementation includes here
/////////////////////////////////////////////////////////

namespace wolf{

inline Eigen::VectorXd ProcessorImu::deltaZero() const
{
    return (Eigen::VectorXd(10) << 0,0,0,  0,0,0,1,  0,0,0 ).finished(); // p, q, v
}

} // namespace wolf

#endif // PROCESSOR_Imu_H
