#include "imu/feature/feature_imu.h"

namespace wolf {

FeatureImu::FeatureImu(const Eigen::VectorXd& _delta_preintegrated,
                       const Eigen::MatrixXd& _delta_preintegrated_covariance,
                       const Eigen::Vector6d& _bias,
                       const Eigen::Matrix<double,9,6>& _dD_db_jacobians,
                       CaptureMotionPtr _cap_imu_ptr) :
    FeatureBase("FeatureImu", _delta_preintegrated, _delta_preintegrated_covariance),
    acc_bias_preint_(_bias.head<3>()),
    gyro_bias_preint_(_bias.tail<3>()),
    jacobian_bias_(_dD_db_jacobians)
{
}

FeatureImu::FeatureImu(CaptureMotionPtr _cap_imu_ptr):
        FeatureBase("FeatureImu", _cap_imu_ptr->getDeltaPreint(), _cap_imu_ptr->getDeltaPreintCov()),
        acc_bias_preint_ (_cap_imu_ptr->getCalibrationPreint().head<3>()),
        gyro_bias_preint_(_cap_imu_ptr->getCalibrationPreint().tail<3>()),
        jacobian_bias_(_cap_imu_ptr->getJacobianCalib())
{
}

FeatureImu::~FeatureImu()
{
    //
}

} // namespace wolf
