#include "imu/sensor/sensor_imu.h"
#include "core/state_block/state_block.h"
#include "core/state_block/state_quaternion.h"

namespace wolf {

SensorImu::SensorImu(const Eigen::VectorXd& _extrinsics, ParamsSensorImuPtr _params) :
        SensorImu(_extrinsics, *_params)
{
    //
}

SensorImu::SensorImu(const Eigen::VectorXd& _extrinsics, const ParamsSensorImu& _params) :
        SensorBase("SensorImu", std::make_shared<StateBlock>(_extrinsics.head(3), true), std::make_shared<StateQuaternion>(_extrinsics.tail(4), true), std::make_shared<StateBlock>(6, false, nullptr), (Eigen::Vector6d()<<_params.a_noise,_params.a_noise,_params.a_noise,_params.w_noise,_params.w_noise,_params.w_noise).finished(), false, false, true),
        a_noise(_params.a_noise),
        w_noise(_params.w_noise),
        ab_initial_stdev(_params.ab_initial_stdev),
        wb_initial_stdev(_params.wb_initial_stdev),
        ab_rate_stdev(_params.ab_rate_stdev),
        wb_rate_stdev(_params.wb_rate_stdev)
{
    assert(_extrinsics.size() == 7 && "Wrong extrinsics vector size! Should be 7 for 2d.");
}

SensorImu::~SensorImu()
{
    //
}

} // namespace wolf

// Register in the SensorFactory
#include "core/sensor/sensor_factory.h"
namespace wolf {
WOLF_REGISTER_SENSOR("SensorImu", SensorImu)
WOLF_REGISTER_SENSOR_AUTO("SensorImu", SensorImu);
} // namespace wolf
