/**
 * \file processor_imu_yaml.cpp
 *
 *  Created on: jan 19, 2017
 *      \author: Dinesh Atchuthan
 */

// wolf yaml
#include "imu/processor/processor_imu.h"
#include "core/yaml/yaml_conversion.h"

// wolf
#include "core/common/factory.h"

// yaml-cpp library
#include <yaml-cpp/yaml.h>

namespace wolf
{

namespace
{
static ProcessorParamsBasePtr createProcessorImuParams(const std::string & _filename_dot_yaml)
{
    YAML::Node config = YAML::LoadFile(_filename_dot_yaml);
    std::cout << _filename_dot_yaml << '\n';

    if (config["type"].as<std::string>() == "ProcessorImu")
    {
        YAML::Node kf_vote = config["keyframe_vote"];

        ProcessorParamsImuPtr params = std::make_shared<ProcessorParamsImu>();
        params->time_tolerance = config["time_tolerance"]           .as<double>();
        params->unmeasured_perturbation_std = config["unmeasured_perturbation_std"].as<double>();

        params->max_time_span       = kf_vote["max_time_span"]      .as<double>();
        params->max_buff_length     = kf_vote["max_buff_length"]    .as<SizeEigen>();
        params->dist_traveled       = kf_vote["dist_traveled"]      .as<double>();
        params->angle_turned        = kf_vote["angle_turned"]       .as<double>();
        params->voting_active       = kf_vote["voting_active"]      .as<bool>();
        params->voting_aux_active   = kf_vote["voting_aux_active"]  .as<bool>();
        return params;
    }

    std::cout << "Bad configuration file. No processor type found." << std::endl;
    return nullptr;
}

// Register in the SensorFactory
const bool WOLF_UNUSED registered_prc_odom_3d = ProcessorParamsFactory::get().registerCreator("ProcessorImu", createProcessorImuParams);

} // namespace [unnamed]

} // namespace wolf
