/**
 * \file sensor_imu_yaml.cpp
 *
 *  Created on: Jan 18, 2017
 *      \author: Dinesh Atchuthan
 */

// wolf yaml
#include "imu/sensor/sensor_imu.h"
#include "core/yaml/yaml_conversion.h"

// wolf
#include "core/common/factory.h"

// yaml-cpp library
#include <yaml-cpp/yaml.h>

namespace wolf
{

namespace
{

static ParamsSensorBasePtr createParamsSensorImu(const std::string & _filename_dot_yaml)
{
    YAML::Node config = YAML::LoadFile(_filename_dot_yaml);

    if (config["type"].as<std::string>() == "SensorImu")
    {
        YAML::Node variances        = config["motion_variances"];
        YAML::Node kf_vote          = config["keyframe_vote"];

        ParamsSensorImuPtr params = std::make_shared<ParamsSensorImu>();

        params->a_noise             = variances["a_noise"]          .as<double>();
        params->w_noise             = variances["w_noise"]          .as<double>();
        params->ab_initial_stdev    = variances["ab_initial_stdev"] .as<double>();
        params->wb_initial_stdev    = variances["wb_initial_stdev"] .as<double>();
        params->ab_rate_stdev       = variances["ab_rate_stdev"]    .as<double>();
        params->wb_rate_stdev       = variances["wb_rate_stdev"]    .as<double>();

        return params;
    }

    std::cout << "Bad configuration file. No sensor type found." << std::endl;
    return nullptr;
}

// Register in the SensorFactory
const bool WOLF_UNUSED registered_imu_intr = ParamsSensorFactory::get().registerCreator("SensorImu", createParamsSensorImu);

} // namespace [unnamed]

} // namespace wolf
